@@ -13,14 +13,14 @@
 # File: pyblueprints/neo4j.py                                       #
 #####################################################################
 
-import neo4jrestclient
+from neo4jrestclient import client 
 from base import Graph
 
 
 class Neo4jGraph(Graph):
 
     def __init__(self, host):
-        self.neograph = neo4jrestclient.GraphDatabase(host)
+        self.neograph = client.GraphDatabase(host)
 
     def addVertex(self, _id=None):
         """Add param declared for compability with the API. Neo4j
@@ -36,7 +36,10 @@ class Neo4jGraph(Graph):
         @params _id: Node unique identifier
 
         @returns The requested Vertex or None"""
-        node = self.neograph.nodes.get(_id)
+        try:
+            node = self.neograph.nodes.get(_id)
+        except client.NotFoundError:
+            return None
         return Vertex(node)
 
     def getVertices(self):
@@ -48,9 +51,8 @@ class Neo4jGraph(Graph):
         @params vertex: Node to be removed"""
         vertex.neoelement.delete()
 
-    def addEdge(self, _id, outVertex, inVertex, label):
+    def addEdge(self, outVertex, inVertex, label):
         """Creates a new edge
-        @params _id: Edge unique identifier
         @params outVertex: Edge origin Vertex
         @params inVertex: Edge target vertex
         @params label: Edge label
@@ -61,6 +63,17 @@ class Neo4jGraph(Graph):
         edge = n1.relationships.create(label, n2)
         return Edge(edge)
 
+    def getEdge(self, _id):
+        """Retrieves an existing edge from the graph
+        @params _id: Edge unique identifier
+
+        @returns The requested Edge or None"""
+        try:
+            edge = self.neograph.relationships.get(_id)
+        except client.NotFoundError:
+            return None
+        return Edge(edge)
+
     def getEdges(self):
         """Returns an iterator with all the vertices"""
         raise NotImplementedError("Method has to be implemented")
@@ -100,7 +113,7 @@ class Element():
         """Returns a set with the property keys of the element
 
         @returns Set of property keys"""
-        return set(self.neoelement.properties.keys())
+        return self.neoelement.properties.keys()
 
     def setProperty(self, key, value):
         """Sets the property of the element to the given value
@@ -114,6 +127,11 @@ class Element():
         @returns The unique identifier of the element"""
         return self.neoelement.id
 
+    def removeProperty(self, key):
+        """Removes the value of the property for the given key
+        @params key: The key which value is being removed"""
+        self.neoelement.properties.pop(key)
+
 
 class Vertex(Element):
     """An abstract class defining a Vertex object representing
@@ -147,6 +165,21 @@ class Vertex(Element):
             for edge in self.neoelement.relationships.incoming():
                 yield Edge(edge)
 
+    def getBothEdges(self, label=None):
+        """Gets all the edges of the node. If label
+        parameter is provided, it only returns the edges of
+        the given label
+        @params label: Optional parameter to filter the edges
+
+        @returns A generator function with the incoming edges"""
+        if label:
+            for edge in self.neoelement.relationships.all(types=[label]):
+                yield Edge(edge)
+        else:
+            for edge in self.neoelement.relationships.all():
+                yield Edge(edge)
+
+
     def __str__(self):
         return "Vertex %s: %s" % (self.neoelement.id,
                                 self.neoelement.properties)
@@ -184,14 +217,14 @@ class Index():
     Index object"""
 
     def __init__(self, indexName, indexClass, indexType, indexObject):
-        if indexClass != "VERTICES" and indexClass != "EDGES":
+        if indexClass != "vertex" and indexClass != "edge":
             raise NameError("%s is not a valid Index Class" % indexClass)
         self.indexClass = indexClass
         self.indexName = indexName
-        if indexType != "AUTOMATIC" and indexType != "MANUAL":
+        if indexType != "automatic" and indexType != "manual":
             raise NameError("%s is not a valid Index Type" % indexType)
         self.indexType = indexType
-        if not isinstance(indexObject, neo4jrestclient.client.Index):
+        if not isinstance(indexObject, client.Index):
             raise TypeError("""%s is not a valid
                             neo4jrestclient.client.Index
                             instance""" \
@@ -214,13 +247,13 @@ class Index():
         return self.indexName
 
     def getIndexClass(self):
-        """Returns the index class (VERTICES or EDGES)
+        """Returns the index class (vertex or edge)
 
         @returns The index class"""
         return self.indexClass
 
     def getIndexType(self):
-        """Returns the index type (AUTOMATIC or MANUAL)
+        """Returns the index type (automatic or manual)
 
         @returns The index type"""
         return self.indexType
@@ -264,43 +297,48 @@ class Neo4jIndexableGraph(Neo4jGraph):
     for indexable graphs"""
 
     def createManualIndex(self, indexName, indexClass):
-        """TODO Documentation"""
-        raise NotImplementedError("Method has to be implemented")
-
-    def createAutomaticIndex(self, name, indexClass):
-        """Creates an index automatically managed my Neo4j
+        """Creates an index manually managed
         @params name: The index name
-        @params indexClass: VERTICES or EDGES
+        @params indexClass: vertex or edge
 
         @returns The created Index"""
-        if indexClass == "VERTICES":
-            index = self.neograph.nodes.indexes.create(name)
-        elif indexClass == "EDGES":
-            index = self.neograph.relationships.indexes.create(name)
+        indexClass = str(indexClass).lower()
+        if indexClass == "vertex":
+            index = self.neograph.nodes.indexes.create(indexName)
+        elif indexClass == "edge":
+            index = self.neograph.relationships.indexes.create(indexName)
         else:
             NameError("Unknown Index Class %s" % indexClass)
-        return Index(name, indexClass, "AUTOMATIC", index)
+        return Index(indexName, indexClass, "automatic", index)
+
+    def createAutomaticIndex(self, indexName, indexClass):
+        """Creates an index automatically managed my Neo4j
+        @params name: The index name
+        @params indexClass: vertex or edge
+
+        @returns The created Index"""
+        raise NotImplementedError("Method has to be implemented")
 
     def getIndex(self, indexName, indexClass):
         """Retrieves an index with a given index name and class
         @params indexName: The index name
-        @params indexClass: VERTICES or EDGES
+        @params indexClass: vertex or edge
 
-        @return The Index object"""
-        if indexClass == "VERTICES":
+        @return The Index object or None"""
+        if indexClass == "vertex":
             try:
-                return Index(indexName, indexClass, "AUTOMATIC",
+                return Index(indexName, indexClass, "manual",
                         self.neograph.nodes.indexes.get(indexName))
-            except neo4jrestclient.request.NotFoundError:
-                raise KeyError("VERTICES index %s not found" % indexName)
-        elif indexClass == "EDGES":
+            except client.NotFoundError:
+                return None
+        elif indexClass == "edge":
             try:
-                return Index(indexName, indexClass, "AUTOMATIC",
+                return Index(indexName, indexClass, "manual",
                         self.neograph.relationships.indexes.get(indexName))
-            except neo4jrestclient.request.NotFoundError:
-                raise KeyError("EDGES index %s not found" % indexName)
+            except client.NotFoundError:
+                return None
         else:
-            raise KeyError("Unknown Index Class (%s). Use VERTICES or EDGES"\
+            raise KeyError("Unknown Index Class (%s). Use vertex or edge"\
                     % indexClass)
 
     def getIndices(self):
@@ -309,60 +347,11 @@ class Neo4jIndexableGraph(Neo4jGraph):
         @returns A generator function over all rhe Index objects"""
         for indexName in self.neograph.nodes.indexes.keys():
             indexObject = self.neograph.nodes.indexes.get(indexName)
-            yield Index(indexName, "VERTICES", "AUTOMATIC", indexObject)
+            yield Index(indexName, "vertex", "manual", indexObject)
         for indexName in self.neograph.relationships.indexes.keys():
             indexObject = self.neograph.relationships.indexes.get(indexName)
-            yield Index(indexName, "EDGES", "AUTOMATIC", indexObject)
+            yield Index(indexName, "edge", "manual", indexObject)
 
     def dropIndex(self, indexName):
         """TODO Documentation"""
         raise NotImplementedError("Method has to be implemented")
-
-
-if __name__ == "__main__":
-    #Creates the object
-    g = Neo4jIndexableGraph('http://localhost:7474/db/data')
-    #Adds a vertex
-    v1 = g.addVertex()
-    #Sets, gets and prints Vertex
-    v1.setProperty('myProp1', 'myValue1')
-    print v1.getPropertyKeys()
-    print v1.getProperty('myProp1')
-    print v1
-    vertexId = v1.getId()
-    print vertexId
-    #Create another vertex and add an Edge
-    v2 = g.addVertex()
-    v2.setProperty('myProp2', 'myValue2')
-    e = g.addEdge(None, v1, v2, 'anEdgeLabel')
-    #Sets, gets and prints Edge
-    e.setProperty('myEdgeProp', 'myEdgeValue')
-    print e.getPropertyKeys()
-    print e.getProperty('myEdgeProp')
-    print e.getInVertex()
-    print e.getOutVertex()
-    print "In edges:"
-    for edge in v2.getInEdges():
-        print edge
-    print "Out edges:"
-    for edge in v2.getOutEdges():
-        print edge
-    #Indexes
-    i1 = g.createAutomaticIndex('testIndex', 'VERTICES')
-    i2 = g.createAutomaticIndex('testIndex2', 'EDGES')
-    for index in g.getIndices():
-        print index
-    i1 = g.getIndex('testIndex', "VERTICES")
-    print i1.getIndexName()
-    print i1.getIndexClass()
-    print i1.getIndexType()
-    i1.put('key1', 'value1', v1)
-    i1.put('key1', 'value1', v2)
-    print i1.count('key1', 'value1')
-    for element in i1.get('key1', 'value1'):
-        print element
-    i1.remove('key1', 'value1', v1)
-    print i1.count('key1', 'value1')
-    g.removeEdge(e)
-    v = g.getVertex(vertexId)
-    g.removeVertex(v1)
